/*

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package components

import (
	"fmt"
	"strings"

	corev1 "k8s.io/api/core/v1"
	networkingv1beta1 "k8s.io/api/networking/v1beta1"
	"k8s.io/apimachinery/pkg/util/intstr"

	meta "k8s.libre.sh/meta"
	ingress "k8s.libre.sh/objects/ingress"
	service "k8s.libre.sh/objects/service"
)

// +kubebuilder:object:generate=true
type Network struct {
	*meta.ObjectMeta `json:"commonMeta,omitempty"`
	Service          Service `json:"service,omitempty"`
	Ingress          Ingress `json:"ingress,omitempty"`
}

// +kubebuilder:object:generate=true
type Service struct {
	*meta.ObjectMeta `json:"meta,omitempty"`
	Port             service.Port       `json:"port,omitempty"`
	Type             corev1.ServiceType `json:"type,omitempty"`
}

// +kubebuilder:object:generate=true
type Ingress struct {
	*meta.ObjectMeta `json:"meta,omitempty"`
	Host             string   `json:"host,omitempty"`
	Paths            []string `json:"routes,omitempty"`
	TLS              bool     `json:"tls,omitempty"`
	TLSSecretRef     string   `json:"tlsSecretRef,omitempty"`
}

func (r *Network) GetIngress() *ingress.Ingress {
	ing := &ingress.Ingress{
		ObjectMeta: r.ObjectMeta,
		IngressSpec: &networkingv1beta1.IngressSpec{
			Rules: r.GetIngressRules(),
			TLS:   r.GetIngressTLS(),
		},
	}
	return ing
}

func (c *Network) GetService() *service.Service {
	svc := &service.Service{
		ObjectMeta: c.ObjectMeta,
		ServiceSpec: &service.ServiceSpec{
			Ports: []service.Port{
				c.Service.Port,
			},
			Type: c.Service.Type,
		},
	}

	return svc
}

func (r *Network) GetIngressAnnotations() map[string]string {
	return r.Ingress.ObjectMeta.Annotations
}

func (r *Network) GetIngressBackendPaths() []networkingv1beta1.HTTPIngressPath {

	bkpaths := []networkingv1beta1.HTTPIngressPath{}

	for _, p := range r.Ingress.Paths {
		path := networkingv1beta1.HTTPIngressPath{
			Path: p,
			Backend: networkingv1beta1.IngressBackend{
				ServiceName: r.ObjectMeta.Name,
				ServicePort: intstr.FromString(r.Service.Port.Name),
			},
		}

		bkpaths = append(bkpaths, path)
	}
	return bkpaths
}

func (r *Network) GetIngressRules() []networkingv1beta1.IngressRule {

	rules := []networkingv1beta1.IngressRule{}

	rule := networkingv1beta1.IngressRule{
		Host: r.Ingress.Host,
		IngressRuleValue: networkingv1beta1.IngressRuleValue{
			HTTP: &networkingv1beta1.HTTPIngressRuleValue{
				Paths: r.GetIngressBackendPaths(),
			},
		},
	}

	rules = append(rules, rule)

	return rules
}

func (r *Network) GetIngressTLS() []networkingv1beta1.IngressTLS {

	tls := networkingv1beta1.IngressTLS{}
	tlsList := []networkingv1beta1.IngressTLS{}

	if &r.Ingress.TLS != nil {
		if len(r.Ingress.TLSSecretRef) == 0 {
			r.Ingress.TLSSecretRef = fmt.Sprintf("%v-%v", strings.Replace(r.Ingress.Host, ".", "-", -1), "tls")
		}
		tls = networkingv1beta1.IngressTLS{
			SecretName: r.Ingress.TLSSecretRef,
		}
		tls.Hosts = append(tls.Hosts, r.Ingress.Host)
		tlsList = append(tlsList, tls)
	} else {
		tlsList = nil
	}
	return tlsList
}

func upsertPath(rules []networkingv1beta1.IngressRule, domain, path string, bk networkingv1beta1.IngressBackend) []networkingv1beta1.IngressRule {
	var rule *networkingv1beta1.IngressRule

	for i := range rules {
		if rules[i].Host == domain {
			rule = &rules[i]
			break
		}
	}

	if rule == nil {
		rules = append(rules, networkingv1beta1.IngressRule{Host: domain})
		rule = &rules[len(rules)-1]
	}

	if rule.HTTP == nil {
		rule.HTTP = &networkingv1beta1.HTTPIngressRuleValue{}
	}

	var httpPath *networkingv1beta1.HTTPIngressPath
	for i := range rule.HTTP.Paths {
		if rule.HTTP.Paths[i].Path == path {
			httpPath = &rule.HTTP.Paths[i]
			break
		}
	}

	if httpPath == nil {
		rule.HTTP.Paths = append(rule.HTTP.Paths, networkingv1beta1.HTTPIngressPath{Path: path})
		httpPath = &rule.HTTP.Paths[len(rule.HTTP.Paths)-1]
	}

	httpPath.Backend = bk

	return rules
}
